library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

entity ButtonFilter_TB is
end ButtonFilter_TB;

architecture TestBench of ButtonFilter_TB is

	component ButtonFilter is
		generic(	C_CLK_FREQ		: integer;
					NUM_SWITCHES	: integer);
		port (	clk				: in	std_logic;
					reset				: in	std_logic;
					buttonArray		: in	std_logic_vector(NUM_SWITCHES-1 downto 0);
					pressArray		: out	std_logic_vector(NUM_SWITCHES-1 downto 0));
	end component;

	constant C_CLK_FREQ		: integer := 50_000_000;
	constant NUM_SWITCHES	: integer := 3;

	signal clk				: std_logic;
	signal reset			: std_logic;
	signal buttonArray	: std_logic_vector(NUM_SWITCHES-1 downto 0);
	signal pressArray		: std_logic_vector(NUM_SWITCHES-1 downto 0);

begin

UUT : ButtonFilter
generic map(C_CLK_FREQ		=> C_CLK_FREQ,
				NUM_SWITCHES	=> NUM_SWITCHES)
port map(	clk				=> clk,
				reset				=> reset,
				buttonArray		=> buttonArray,
				pressArray		=> pressArray);

MainClock : process
begin
	clk <= '1';
	wait for 10 ns;
	clk <= '0';
	wait for 10 ns;
end process;

TB : process
begin

	reset <= '1';
	buttonArray <= b"111";	-- Released

	wait until (clk'event and clk='1');
	reset <= '0';

	wait for 1 us;

	buttonArray <= b"110";	-- Pressed
	wait for 1 ms;

	buttonArray <= b"111";	-- Released
	wait for 5 ms;

	buttonArray <= b"110";	-- Pressed
	wait for 2 ms;

	buttonArray <= b"111";	-- Released
	wait for 2 ms;

	buttonArray <= b"110";	-- Pressed

	wait;

end process;

end TestBench;
