library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity CellArray is
	port (	clk					: in	std_logic;
				reset					: in	std_logic;

				dIn					: in	std_logic_vector(3 downto 0);
				dOut					: out	std_logic_vector(1 downto 0);

				configCellNum		: in	integer range 0 to 31;
				config				: in	std_logic_vector(11 downto 0);
				configWEn			: in	std_logic;

				outputSel0			: in	integer range 0 to 31;
				outputSel1			: in	integer range 0 to 31;
				outputSelWEn		: in	std_logic;

				failModeCellNum	: in	integer range 0 to 31;
				failMode				: in	std_logic_vector(1 downto 0);
				failModeWEn			: in	std_logic);

end CellArray;

architecture Arch of CellArray is

	component Cell is
	port (	clk				: in	std_logic;
				reset				: in	std_logic;
				dIn				: in	std_logic_vector(15 downto 0);
				dOut				: out	std_logic;
				config			: in	std_logic_vector(11 downto 0);
				configWEn		: in	std_logic;
				failMode			: in	std_logic_vector(1 downto 0);
				failModeWEn		: in	std_logic);
	end component;

 	type COLUMN_OUTPUTS_TYPE is array (0 to 3) of std_logic_vector(7 downto 0);
	signal cellDOut : COLUMN_OUTPUTS_TYPE;

 	type ARRAY_OUTPUTS_TYPE is array (0 to 31) of std_logic;
	signal arrayDOut : ARRAY_OUTPUTS_TYPE;

	signal output0		: integer range 0 to 31;
	signal output1		: integer range 0 to 31;

begin

-- ArrayGenerate ----------------------------------------------------------------------------------------------
-- Create the array fo cells
---------------------------------------------------------------------------------------------------------------
ColGenerate : for colNum in 0 to 3 generate

	signal cellDIn : std_logic_vector(15 downto 0);

begin

	-- If first column
	Connect1 : if (colNum=0) generate
		cellDIn(3 downto 0) <= dIn;
		cellDIn(7 downto 4) <= not dIn;
		cellDIn(15 downto 8) <= (others => '0');
	end generate;

	-- If other columns
	Connect2 : if (colNum>0) generate
		cellDIn(3 downto 0) <= dIn;
		cellDIn(7 downto 4) <= not dIn;
		cellDIn(15 downto 8) <= cellDOut(colNum-1);
	end generate;

	RowGenerate : for rowNum in 0 to 7 generate

		constant cellNumber		: integer := rowNum + (colNum*8);
		signal cellConfigWEn		: std_logic;
		signal cellFailModeWEn	: std_logic;

	begin

		cellConfigWEn		<= '1' when (configCellNum=cellNumber and configWEn='1') else '0';
		cellFailModeWEn	<= '1' when (failModeCellNum=cellNumber and failModeWEn='1') else '0';

		CellInst : Cell
		port map (	clk				=> clk,
						reset				=> reset,
						dIn				=> cellDIn,
						dOut				=> cellDOut(colNum)(rowNum),
						config			=> config,
						configWEn		=> cellConfigWEn,
						failMode			=> failMode,
						failModeWEn		=> cellFailModeWEn);

		OutputLatch : process(clk)
		begin
			if (clk'event and clk='1') then
				if (reset='1') then
					arrayDOut((colNum*8) + rowNum) <= '0';
				else
					arrayDOut((colNum*8) + rowNum) <= cellDOut(colNum)(rowNum);
				end if;
			end if;
		end process OutputLatch;

	end generate;

end generate;

OutputSelectionWrite : process(clk)
begin
	if (clk'event and clk='1') then
		if (reset='1') then
			output0 <= 0;
			output1 <= 0;

		elsif (outputSelWEn = '1') then
			output0 <= outputSel0;
			output1 <= outputSel1;

		end if;
	end if;
end process OutputSelectionWrite;

dOut(0) <= arrayDOut(output0);
dOut(1) <= arrayDOut(output1);

end Arch;
