library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

entity opb_ftblock_TB is
end opb_ftblock_TB;

architecture TestBench of opb_ftblock_TB is

	component opb_ftblock is
		generic
		(
			C_BASEADDR		: std_logic_vector	:= X"00000000";
			C_HIGHADDR		: std_logic_vector	:= X"0000000F";
			C_OPB_AWIDTH	: integer				:= 32;
			C_OPB_DWIDTH	: integer				:= 32;
			C_FAMILY			: string					:= "virtex2p"
		);
		port
		(
			ledPort			: out	std_logic_vector(2 downto 0);
			OPB_Clk			: in	std_logic;
			OPB_Rst			: in	std_logic;
			Sl_DBus			: out	std_logic_vector(0 to C_OPB_DWIDTH-1);
			Sl_errAck		: out	std_logic;
			Sl_retry			: out	std_logic;
			Sl_toutSup		: out	std_logic;
			Sl_xferAck		: out	std_logic;
			OPB_ABus			: in	std_logic_vector(0 to C_OPB_AWIDTH-1);
			OPB_BE			: in	std_logic_vector(0 to C_OPB_DWIDTH/8-1);
			OPB_DBus			: in	std_logic_vector(0 to C_OPB_DWIDTH-1);
			OPB_RNW			: in	std_logic;
			OPB_select		: in	std_logic;
			OPB_seqAddr		: in	std_logic
		);
	end component;

	constant C_BASEADDR		: std_logic_vector := x"0000_0000";
	constant C_OPB_AWIDTH	: integer := 32;
	constant C_OPB_DWIDTH	: integer := 32;

	signal ledPort			: std_logic_vector(2 downto 0);
	signal OPB_Clk			: std_logic;
	signal OPB_Rst			: std_logic;
	signal Sl_DBus			: std_logic_vector(0 to C_OPB_DWIDTH-1);
	signal Sl_errAck		: std_logic;
	signal Sl_retry		: std_logic;
	signal Sl_toutSup		: std_logic;
	signal Sl_xferAck		: std_logic;
	signal OPB_ABus		: std_logic_vector(0 to C_OPB_AWIDTH-1);
	signal OPB_BE			: std_logic_vector(0 to C_OPB_DWIDTH/8-1);
	signal OPB_DBus		: std_logic_vector(0 to C_OPB_DWIDTH-1);
	signal OPB_RNW			: std_logic;
	signal OPB_select		: std_logic;
	signal OPB_seqAddr	: std_logic;

	function reverseVector (a: in std_logic_vector) return std_logic_vector is
		variable result: std_logic_vector(a'reverse_range);
	begin
		for i in a'range loop
			result((a'length-1)-i) := a(i);
 		end loop;
		return result;
	end;

begin

UUT : opb_ftblock
generic map (	C_BASEADDR		=> C_BASEADDR,
					C_HIGHADDR		=> X"0000000F",
					C_OPB_AWIDTH	=> C_OPB_AWIDTH,
					C_OPB_DWIDTH	=> C_OPB_DWIDTH,
					C_FAMILY			=> "virtex2p")
port map		(	ledPort			=> ledPort,
					OPB_Clk			=> OPB_Clk,
					OPB_Rst			=> OPB_Rst,
					Sl_DBus			=> Sl_DBus,
					Sl_errAck		=> Sl_errAck,
					Sl_retry			=> Sl_retry,
					Sl_toutSup		=> Sl_toutSup,
					Sl_xferAck		=> Sl_xferAck,
					OPB_ABus			=> OPB_ABus,
					OPB_BE			=> OPB_BE,
					OPB_DBus			=> OPB_DBus,
					OPB_RNW			=> OPB_RNW,
					OPB_select		=> OPB_select,
					OPB_seqAddr		=> OPB_seqAddr);

MainClock : process
begin
	OPB_Clk <= '1';
	wait for 10 ns;
	OPB_Clk <= '0';
	wait for 10 ns;
end process;

TB : process

	function selNumsToVector (sel1 : integer; sel0 : integer) return std_logic_vector is
		variable val : std_logic_vector(31 downto 0) := (others => '0');
	begin
		val(9 downto 0) := std_logic_vector(to_unsigned(sel1, 5)) & std_logic_vector(to_unsigned(sel0, 5));
		return val;
	end;

	procedure opbWrite(baseAddress : std_logic_vector(0 to 31); reg : integer; data : std_logic_vector(31 downto 0) ) is
		variable add : unsigned(31 downto 0);
	begin
		add := to_unsigned((reg*4), 31) + unsigned(reverseVector(baseAddress));
		OPB_ABus <= reverseVector(std_logic_vector(add));
		OPB_DBus	<= reverseVector(data);
		OPB_RNW		<= '0';	-- write
		OPB_select	<= '1';
		wait until (OPB_Clk'event and OPB_Clk='1');
		OPB_select	<= '0';
		wait until (OPB_Clk'event and OPB_Clk='1');
	end procedure opbWrite;

	variable s1, s2		: positive;
	variable randNum		: real;
	variable randConfig	: std_logic_vector(11 downto 0);

begin

	OPB_Rst		<= '1';
	OPB_select	<= '0';

	wait until (OPB_Clk'event and OPB_Clk='1');
	OPB_Rst <= '0';

	wait until (OPB_Clk'event and OPB_Clk='1');
	wait until (OPB_Clk'event and OPB_Clk='1');

	--	Set Input Source to External vector
	opbWrite(C_BASEADDR, 7, x"0000_0007");

	-- Load Target RAM
	opbWrite(C_BASEADDR, 8, x"0000_0000");	-- Set Vector to 0
	opbWrite(C_BASEADDR, 9, x"0000_000" & b"0010");	-- 0000
	opbWrite(C_BASEADDR, 9, x"0000_000" & b"0001");	-- 0001
	opbWrite(C_BASEADDR, 9, x"0000_000" & b"0001");	-- 0010
	opbWrite(C_BASEADDR, 9, x"0000_000" & b"0010");	-- 0011
	opbWrite(C_BASEADDR, 9, x"0000_000" & b"0001");	-- 0100
	opbWrite(C_BASEADDR, 9, x"0000_000" & b"0010");	-- 0101
	opbWrite(C_BASEADDR, 9, x"0000_000" & b"0010");	-- 0110
	opbWrite(C_BASEADDR, 9, x"0000_000" & b"0001");	-- 0111
	opbWrite(C_BASEADDR, 9, x"0000_000" & b"0001");	-- 1000
	opbWrite(C_BASEADDR, 9, x"0000_000" & b"0010");	-- 1001
	opbWrite(C_BASEADDR, 9, x"0000_000" & b"0010");	-- 1010
	opbWrite(C_BASEADDR, 9, x"0000_000" & b"0001");	-- 1011
	opbWrite(C_BASEADDR, 9, x"0000_000" & b"0010");	-- 1100
	opbWrite(C_BASEADDR, 9, x"0000_000" & b"0001");	-- 1101
	opbWrite(C_BASEADDR, 9, x"0000_000" & b"0001");	-- 1110
	opbWrite(C_BASEADDR, 9, x"0000_000" & b"0010");	-- 1111

	-----------------------------------------------
	-- Array 0
	-----------------------------------------------

	-- Set Config Array Num
	opbWrite(C_BASEADDR, 0, x"0000_0000");

	-- Set Config Cell Num
	opbWrite(C_BASEADDR, 1, x"0000_0000");

	-- Load Config
	opbWrite(C_BASEADDR, 2, x"0000049B");
	opbWrite(C_BASEADDR, 2, x"000008FD");
	opbWrite(C_BASEADDR, 2, x"00000AA2");
	opbWrite(C_BASEADDR, 2, x"0000046A");
	opbWrite(C_BASEADDR, 2, x"00000A1A");
	opbWrite(C_BASEADDR, 2, x"00000F69");
	opbWrite(C_BASEADDR, 2, x"000001D8");
	opbWrite(C_BASEADDR, 2, x"000007CF");
	opbWrite(C_BASEADDR, 2, x"000001D0");
	opbWrite(C_BASEADDR, 2, x"00000C52");
	opbWrite(C_BASEADDR, 2, x"00000726");
	opbWrite(C_BASEADDR, 2, x"000008C6");
	opbWrite(C_BASEADDR, 2, x"00000C16");
	opbWrite(C_BASEADDR, 2, x"00000BF4");
	opbWrite(C_BASEADDR, 2, x"000002A0");
	opbWrite(C_BASEADDR, 2, x"0000023D");
	opbWrite(C_BASEADDR, 2, x"000003A4");
	opbWrite(C_BASEADDR, 2, x"0000004F");
	opbWrite(C_BASEADDR, 2, x"00000AB6");
	opbWrite(C_BASEADDR, 2, x"000009B3");
	opbWrite(C_BASEADDR, 2, x"000006B6");
	opbWrite(C_BASEADDR, 2, x"0000093D");
	opbWrite(C_BASEADDR, 2, x"00000CE2");
	opbWrite(C_BASEADDR, 2, x"00000205");
	opbWrite(C_BASEADDR, 2, x"00000E0E");
	opbWrite(C_BASEADDR, 2, x"00000DF3");
	opbWrite(C_BASEADDR, 2, x"00000C39");
	opbWrite(C_BASEADDR, 2, x"00000CD3");
	opbWrite(C_BASEADDR, 2, x"00000A5D");
	opbWrite(C_BASEADDR, 2, x"00000983");
	opbWrite(C_BASEADDR, 2, x"0000012F");
	opbWrite(C_BASEADDR, 2, x"0000042C");

	--	Set Output Selection
	opbWrite(C_BASEADDR, 3, selNumsToVector(18, 26));

	-----------------------------------------------
	-- Array 1
	-----------------------------------------------

	-- Set Config Array Num
	opbWrite(C_BASEADDR, 0, x"0000_0001");

	-- Set Config Cell Num
	opbWrite(C_BASEADDR, 1, x"0000_0000");

	-- Load Config
	opbWrite(C_BASEADDR, 2, x"000005FC");
	opbWrite(C_BASEADDR, 2, x"0000069D");
	opbWrite(C_BASEADDR, 2, x"00000079");
	opbWrite(C_BASEADDR, 2, x"000007A9");
	opbWrite(C_BASEADDR, 2, x"000002F9");
	opbWrite(C_BASEADDR, 2, x"00000129");
	opbWrite(C_BASEADDR, 2, x"000003C8");
	opbWrite(C_BASEADDR, 2, x"0000006D");
	opbWrite(C_BASEADDR, 2, x"000009C6");
	opbWrite(C_BASEADDR, 2, x"00000C04");
	opbWrite(C_BASEADDR, 2, x"00000884");
	opbWrite(C_BASEADDR, 2, x"00000738");
	opbWrite(C_BASEADDR, 2, x"000005A4");
	opbWrite(C_BASEADDR, 2, x"00000C06");
	opbWrite(C_BASEADDR, 2, x"00000DA9");
	opbWrite(C_BASEADDR, 2, x"00000579");
	opbWrite(C_BASEADDR, 2, x"000003D2");
	opbWrite(C_BASEADDR, 2, x"00000A48");
	opbWrite(C_BASEADDR, 2, x"00000EAD");
	opbWrite(C_BASEADDR, 2, x"00000FD9");
	opbWrite(C_BASEADDR, 2, x"00000E7B");
	opbWrite(C_BASEADDR, 2, x"000000EF");
	opbWrite(C_BASEADDR, 2, x"00000A9D");
	opbWrite(C_BASEADDR, 2, x"00000203");
	opbWrite(C_BASEADDR, 2, x"00000321");
	opbWrite(C_BASEADDR, 2, x"00000A57");
	opbWrite(C_BASEADDR, 2, x"0000050A");
	opbWrite(C_BASEADDR, 2, x"0000033E");
	opbWrite(C_BASEADDR, 2, x"00000F57");
	opbWrite(C_BASEADDR, 2, x"00000938");
	opbWrite(C_BASEADDR, 2, x"000005D6");
	opbWrite(C_BASEADDR, 2, x"00000500");

	--	Set Output Selection
	opbWrite(C_BASEADDR, 3, selNumsToVector(19, 14));

	-----------------------------------------------
	-- Array 2
	-----------------------------------------------

	-- Set Config Array Num
	opbWrite(C_BASEADDR, 0, x"0000_0002");

	-- Set Config Cell Num
	opbWrite(C_BASEADDR, 1, x"0000_0000");

	-- Load Config
	opbWrite(C_BASEADDR, 2, x"00000662");
	opbWrite(C_BASEADDR, 2, x"00000771");
	opbWrite(C_BASEADDR, 2, x"0000036E");
	opbWrite(C_BASEADDR, 2, x"0000058F");
	opbWrite(C_BASEADDR, 2, x"000003E6");
	opbWrite(C_BASEADDR, 2, x"00000369");
	opbWrite(C_BASEADDR, 2, x"000008E1");
	opbWrite(C_BASEADDR, 2, x"00000C7C");
	opbWrite(C_BASEADDR, 2, x"00000505");
	opbWrite(C_BASEADDR, 2, x"000005E2");
	opbWrite(C_BASEADDR, 2, x"000005BC");
	opbWrite(C_BASEADDR, 2, x"00000D8F");
	opbWrite(C_BASEADDR, 2, x"000000D6");
	opbWrite(C_BASEADDR, 2, x"00000239");
	opbWrite(C_BASEADDR, 2, x"00000D7F");
	opbWrite(C_BASEADDR, 2, x"0000099E");
	opbWrite(C_BASEADDR, 2, x"00000A90");
	opbWrite(C_BASEADDR, 2, x"0000003C");
	opbWrite(C_BASEADDR, 2, x"000003BD");
	opbWrite(C_BASEADDR, 2, x"00000E5E");
	opbWrite(C_BASEADDR, 2, x"00000857");
	opbWrite(C_BASEADDR, 2, x"00000D06");
	opbWrite(C_BASEADDR, 2, x"000001C6");
	opbWrite(C_BASEADDR, 2, x"00000CA5");
	opbWrite(C_BASEADDR, 2, x"0000006D");
	opbWrite(C_BASEADDR, 2, x"00000D59");
	opbWrite(C_BASEADDR, 2, x"000000BF");
	opbWrite(C_BASEADDR, 2, x"00000A8C");
	opbWrite(C_BASEADDR, 2, x"0000066D");
	opbWrite(C_BASEADDR, 2, x"0000035B");
	opbWrite(C_BASEADDR, 2, x"00000450");
	opbWrite(C_BASEADDR, 2, x"000009C5");

	--	Set Output Selection
	opbWrite(C_BASEADDR, 3, selNumsToVector(25, 22));
--	opbWrite(C_BASEADDR, 3, selNumsToVector(25, 21));

	-- Loop through vectors
	for vector in 0 to 15 loop
		opbWrite(C_BASEADDR, 8, std_logic_vector(to_unsigned(vector,32)));
		wait until (OPB_Clk'event and OPB_Clk='1');
		wait until (OPB_Clk'event and OPB_Clk='1');
		wait until (OPB_Clk'event and OPB_Clk='1');
		wait until (OPB_Clk'event and OPB_Clk='1');
		wait until (OPB_Clk'event and OPB_Clk='1');
		wait until (OPB_Clk'event and OPB_Clk='1');
	end loop;

	--	Set Input Source to Internal vector
	opbWrite(C_BASEADDR, 7, x"0000_0000");

	wait for 500 ns;
	wait until (OPB_Clk'event and OPB_Clk='1');

	-- Set FailMode Array Num
	opbWrite(C_BASEADDR, 4, x"0000_0002");

	-- Set FailMode Cell Num
	opbWrite(C_BASEADDR, 5, x"0000_0019");

	-- Set FailMode
	opbWrite(C_BASEADDR, 6, x"0000_0001");

	wait;

end process;

	-- Load randomm array config
--	for vector in 0 to 31 loop
--		UNIFORM(s1, s2, randNum);
--		randConfig := std_logic_vector(to_unsigned(integer(randNum * 4096.0), 12));
--		opbWrite(C_BASEADDR, 2, x"00000" & randConfig);
--	end loop;

end TestBench;
